
perform_canopy_cleanup <- function(canopy_data_frame, fieldName, rowOrientation, sensorOffset, speedRange, ndviRange, summaryFile) {

  ## Control Values for automated Process ## These need to be set on the web interface
  FieldName <- fieldName
  DIR <- rowOrientation # if there are blocks with different orientations then select -1 to avoid triming on orientation
  # Distance interrow
  # Number of rows in a machine (sensor) operation (Swath width)
  OffSet <- sensorOffset # displacement of GPS from the sensed vines, POSITIVE OFFSET = rightfacing sensor (imaging to right of the GPS location)
  # SensorNo<-2                       # either 1 or 2 sensor system
  Date <- Sys.Date()
  # removal of outliers happens in the geodatabase, so does not need to be coded
  NDVIrange1 <- ndviRange # range of expected values for the NDVI response (must be 0 -1) of Sensor 1
  NDVIrange2 <- ndviRange # range of expected values for the NDVI response (must be 0 -1) of Sensor 2 (not used if SensorNo == 1)
  SpdRange <- speedRange # the expected variation in speed - for tractors 1 is usually sufficient, for ATVs a value of 2 or 3 depending on the operator.

  # remove implausible NDVI values
  temp.df <- subset(canopy_data_frame, NDVI > 0 & NDVI < 1)

  temp.df.tr <- temp.df

  ## temp.df.tr is the output at this stage, to Eastings/Northings (cartesian coordinates), 1 Hz data and with NDVI restricted to (0,1)
  # Backup file created for the deleted points analysis later on
  temp.df2 <- temp.df


  ## Trim data of outliers/inliers ###

  ### CLEAN UP OF EXTREME VALUES ###

  ### Remove Strange reflectane values ###
  if ("RE" %in% colnames(temp.df.tr@data)) {
    temp.df.tr <- subset(temp.df.tr, RE > 0 & RE < 100)
    temp.df.tr <- subset(temp.df.tr, Red > 0 & Red < 100)
    temp.df.tr <- subset(temp.df.tr, NIR > 0 & NIR < 100)
  }

  if ("Sensor" %in% colnames(temp.df.tr@data)) {
    temp.df.trA <- subset(temp.df.tr, Sensor == 1 & NDVI > NDVIrange1[1] & NDVI < NDVIrange1[2])
    temp.df.trB <- subset(temp.df.tr, Sensor == 2 & NDVI > NDVIrange2[1] & NDVI < NDVIrange2[2])
    temp.df.tr <- rbind(temp.df.trA, temp.df.trB)
    rm(temp.df.trA, temp.df.trB)
  } else {
    temp.df.tr <- subset(temp.df.tr, NDVI > NDVIrange1[1] & NDVI < NDVIrange1[2])
  }

  ### Trim points at high or low speed (relative to mean) ###
  if ("Speed" %in% colnames(temp.df.tr@data)) {
    temp.df.tr <- subset(temp.df.tr, Speed > 0.2)
    temp.df.tr <- subset(temp.df.tr, Speed > (median(temp.df$Speed - SpdRange)) & Speed < (median(temp.df$Speed + SpdRange))) ## = speed > 3 for mph and < 7 for mph
  }

  ### Trim points not in same orientation as rows ### Row Direction set in the Parameters
  if ("Course" %in% colnames(temp.df.tr@data)) {

    ### degree to radians, add +/- then backtransform to degrees
    atemp <- (DIR * (pi / 180) - (pi / 12)) * (180 / pi)
    btemp <- (DIR * (pi / 180) + (pi / 12)) * (180 / pi)
    ctemp <- (DIR * (pi / 180) - (pi / 12) + pi) * (180 / pi)
    dtemp <- (DIR * (pi / 180) + (pi / 12) + pi) * (180 / pi)

    if (atemp < 0) atemp <- 360 - abs(atemp) else atemp <- atemp
    if (atemp > 360) atemp <- atemp - 360 else atemp <- atemp
    if (btemp < 0) btemp <- 360 - abs(btemp) else btemp <- btemp
    if (btemp > 360) btemp <- btemp - 360 else btemp <- btemp
    if (ctemp < 0) ctemp <- 360 - abs(ctemp) else ctemp <- ctemp
    if (ctemp > 360) ctemp <- ctemp - 360 else ctemp <- ctemp
    if (dtemp < 0) dtemp <- 360 - abs(dtemp) else dtemp <- dtemp
    if (dtemp > 360) dtemp <- dtemp - 360 else dtemp <- dtemp

    temp.df.tr.N <- as.data.frame(temp.df.tr)
    if (atemp < btemp) temp.df.tr.N <- subset(temp.df.tr.N, Course > atemp & Course < btemp) else temp.df.tr.N <- subset(temp.df.tr.N, Course < btemp | Course > atemp)

    temp.df.tr.S <- as.data.frame(temp.df.tr)
    if (ctemp < dtemp) temp.df.tr.S <- subset(temp.df.tr.S, Course > ctemp & Course < dtemp) else temp.df.tr.S <- subset(temp.df.tr.S, Course < dtemp | Course > ctemp)

    ## Adjust the GPS location based on direction - CHECK MOUNTING DIRECTION - POSITIVE OFFSET = rightfacing

    # North #
    Eastsadj <- temp.df.tr.N$Eastings + (abs(cos(DIR * (pi / 180))) * OffSet)
    Northsadj <- temp.df.tr.N$Northings + (abs(sin(DIR * (pi / 180))) * OffSet)
    temp.df.tr.N <- cbind(temp.df.tr.N, Eastsadj, Northsadj)

    # South #
    Eastsadj <- temp.df.tr.S$Eastings - (abs(cos(DIR * (pi / 180))) * OffSet)
    Northsadj <- temp.df.tr.S$Northings - (abs(sin(DIR * (pi / 180))) * OffSet)
    temp.df.tr.S <- cbind(temp.df.tr.S, Eastsadj, Northsadj)


    temp.df.tr2 <- rbind(temp.df.tr.N, temp.df.tr.S)
    temp.df.tr2 <- subset(temp.df.tr2, select = c(-Eastings, -Northings))
    setnames(temp.df.tr2, old = c("Eastsadj", "Northsadj"), new = c("Eastings", "Northings"))

    coordinates(temp.df.tr2) <- c("Eastings", "Northings")
    proj4string(temp.df.tr2) <- proj4string(temp.df.tr)
  }

  if (DIR == -1) temp.df.tr2 <- temp.df.tr # not possible to do this with multiple row orientations.

  # Generate a dataframe of trimmed data that was removed for a plot  ##  NB Replicated values likely but this is only for a visual assessment so reps are no problem.
  temp.df.del <- anti_join(as.data.frame(temp.df2), as.data.frame(temp.df.tr))


  # =Generates a separate window with multiple summary plots and exports it as a pdf file.
  # Summary plots should be used to adjust input variables till suitable solution is found for data trimming.
  # Different plots are generated for the 1 and 2 Sensor system

  if(missing(summaryFile))
    summaryFile <- tempfile(pattern = "cleanup-summary-", fileext = ".pdf")
  pdf(file = summaryFile)

  tryCatch({
    if ("Sensor" %in% colnames(temp.df.tr@data)) {
      par(font.axis = 1, font.lab = 1, font.main = 2, family = "sans", oma = c(0, 0, 3, 0))
      layout(matrix(c(1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 5, 5, 5, 6, 6, 6, 7, 7, 7, 4, 4, 4, 5, 5, 5, 6, 6, 6, 7, 7, 7, 8, 8, 8, 8, 8, 8, 9, 9, 9, 10, 10, 10, 8, 8, 8, 8, 8, 8, 9, 9, 9, 10, 10, 10), 7, 12, byrow = TRUE))
      temp.df.tr2_s1 <- subset(temp.df.tr2, Sensor == 1)
      temp.df.tr2_s2 <- subset(temp.df.tr2, Sensor == 2)
      temp.df_s1 <- subset(temp.df, Sensor == 1)
      temp.df_s2 <- subset(temp.df, Sensor == 2)

      # plot(temp.df$Longitude,temp.df$Latitude, xlab="Longitude", ylab="Latitude", main="Raw Data (Geo)",mtext(FieldName, side=3, font=1))
      plot(temp.df$Eastings, temp.df$Northings, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Raw Data Points", pch = 20, cex = .5)
      plot(temp.df.tr2$Eastings, temp.df.tr2$Northings, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Trimmed Data Points", pch = 20, cex = .5)
      plot(temp.df.del$Eastings, temp.df.del$Northings, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Removed Points", pch = 20, cex = .5)
      hist(temp.df_s1$Speed, xlab = "", ylab = "", main = "Sens 1 Raw Speed")
      hist(temp.df.tr2_s1$Speed, xlab = "", ylab = "", main = "Sens 1 Speed Trim")
      hist(temp.df_s1$NDVI, xlab = "", ylab = "", main = "Sens 1 Raw NDVI")
      hist(temp.df.tr2_s1$NDVI, xlab = "", ylab = "", main = "Sens 1 NDVI Trim")
      NDVIhist <- subset(temp.df, NDVI > 0 & NDVI < 1)
      hist(NDVIhist$NDVI, xlab = "", ylab = "", main = "NDVI Range All Sensors")
      hist(temp.df_s2$NDVI, xlab = "", ylab = "", main = "Sens 2 Raw NDVI")
      hist(temp.df.tr2_s2$NDVI, xlab = "", ylab = "", main = "Sens 2 NDVI Trim")
      mtext(paste(FieldName, " - ", Date), outer = TRUE, cex = 1.5)
    } else {
      par(font.axis = 1, font.lab = 1, font.main = 2, family = "sans", oma = c(0, 0, 3, 0))
      layout(matrix(c(1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 5, 5, 5, 6, 6, 6, 7, 7, 7), 2, 12, byrow = TRUE))

      # plot(temp.df$Longitude,temp.df$Latitude, xlab="Longitude", ylab="Latitude", main="Raw Data (Geo)",mtext(FieldName, side=3, font=1))
      plot(temp.df$Eastings, temp.df$Northings, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Raw Data Points", pch = 20, cex = .5)
      plot(temp.df.tr$Eastings, temp.df.tr$Northings, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Trimmed Data Points", pch = 20, cex = .5)
      plot(temp.df.del$Eastings, temp.df.del$Northings, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Removed Points", pch = 20, cex = .5)
      hist(temp.df$Speed, xlab = "", ylab = "", main = "Speed (Raw)")
      hist(temp.df.tr2$Speed, xlab = "", ylab = "", main = "Speed Trimmed")
      temp.df2 <- subset(temp.df, NDVI > -0.2 & NDVI < 1.2)
      hist(temp.df2$NDVI, xlab = "", ylab = "", main = "NDVI (Raw)")
      rm(temp.df2)
      hist(temp.df.tr2$NDVI, xlab = "", ylab = "", main = "NDVI Trimmed")
      mtext(paste(FieldName, " - ", Date), outer = TRUE, cex = 1.5)
    }
  },
  error = function(error) stop(error$message),
  finally = dev.off()
  )

  # build removed SpatialPointsDataFrame
  coordinates(temp.df.del) <- c("Eastings", "Northings")
  proj4string(temp.df.del) <- proj4string(canopy_data_frame)

  # select sensor subset
  if ("Sensor" %in% colnames(temp.df.tr2@data)) {
      df_sensor1 <- subset(temp.df.tr2, Sensor == 1)
      df_sensor2 <- subset(temp.df.tr2, Sensor == 2)
      mean_sensor1 <- mean(df_sensor1@data$NDVI)
      mean_sensor2 <- mean(df_sensor2@data$NDVI)
      var_sensor1 <- var(df_sensor1@data$NDVI)
      var_sensor2 <- var(df_sensor2@data$NDVI)
      if(mean_sensor1 > 0.6 && mean_sensor2 > 0.6) {
          if(var_sensor1 > var_sensor2) {
              df_sensor <-df_sensor1
          } else {
              df_sensor <-df_sensor2
          }
      } else if (mean_sensor1 > mean_sensor2) {
          df_sensor <-df_sensor1 
      } else {
          df_sensor <-df_sensor2 
      }
  } else 
    df_sensor <- temp.df.tr2

  list(summaryFile = summaryFile, trimmedDataFrame = df_sensor, removedDataFrame = temp.df.del)
}
