get_canopy_data_frame <- function(canopy_file, crs) {
  data <- read.csv(file = canopy_file, header = TRUE, sep = ",", stringsAsFactors = FALSE)
  
  if (ncol(data)==3) {
    colnames(data) <- c("Eastings", "Northings", "NDVI")
  } 
  
  else if (ncol(data)==12) {
    colnames(data) <- c("Eastings", "Northings", "Elevation", "Fix_Type", "UTC_Time", "Speed", "Course", "RE", "NIR", "Red", "NDRE", "NDVI")
    data <- subset(data, select = -Fix_Type)
    data <- aggregate(data, list(data$UTC_Time), mean)
    data <- data[,2:ncol(data)]
  } 
  
  else if (ncol(data)==14) {
    colnames(data) <- c("Eastings", "Northings", "Elevation", "Fix_Type", "UTC_Time", "Speed", "Course", "Sensor", "NDRE", "NDVI", "RE", "NIR", "Red", "AppRate")
    data <- subset(data, select = c(-AppRate, -Fix_Type))
  }
  
  else if (ncol(data)==16) {
    colnames(data) <- c("Northings","Eastings","Course","Speed","Elevation","HDOP","Fix_Type","UTC_Date","UTC_Time","Sensor","NDRE","NDVI","RE","NIR","Red","AppRate")
    data <- subset(data, select = c(-AppRate, -HDOP, -Fix_Type))
  } 
  
  else if (ncol(data)==18) {
    colnames(data) <- c("Northings","Eastings","Course","Speed","Elevation","HDOP","Fix_Type","UTC_Date","UTC_Time","INT_Volt","EXT_VOLT","Sensor","NDRE","NDVI","RE","NIR","Red","AppRate")
    data <- subset(data, select = c(-AppRate, -HDOP, -Fix_Type))
  } else {
    stop("Unknown canopy file format")
  }
  
  # Ensure the Lat/Long and Sensor cols contain no strange values All columns as numeric.
  for (d in 1:ncol(data)) {
    data[, d] <- as.numeric(as.character(data[, d]))
  }
  
  # All rows with missing/errorneous data are removed.
  for (d in 1:ncol(data)) {
    data <- subset(data, data[, d] != "NA")
  }
  
  coordinates(data) <- c("Eastings", "Northings")
  proj4string(data) <- CRS("+proj=longlat +datum=WGS84")
  
  if(missing(crs))
      crs <- get_epsg_crs(c(data$Eastings[1], data$Northings[1]))
  spTransform(data, crs)
}