
sampling <- function(temp.shp, Canopy, rowwidth, rowOrientation, SampSizeSpec, maxRowSampled, bufferRows, bufferVines) {

  # SUBROUTINE TO CREATE ALIGNED ROWS

  bnd.shp.bnd <- temp.shp@bbox
  # Boundary box enlarged - 500 m here.
  bnd.shp.bnd.expand <- c(bnd.shp.bnd[, 1] - 100, bnd.shp.bnd[, 2] + 100)
  S <- bnd.shp.bnd.expand
  coords.bb <- matrix(c(S[1], S[2], S[1], S[4], S[3], S[2], S[3], S[4]), ncol = 2, byrow = TRUE)
  coords.bb <- data.frame(coords.bb)
  colnames(coords.bb) <- c("X", "Y")
  coordinates(coords.bb) <- ~ X + Y
  # generate a grid around the boundary that is enlarged
  temp.grid <- makegrid(coords.bb, cellsize = rowwidth)
  coordinates(temp.grid) <- ~ x1 + x2
  temp.grid2 <- points2grid(temp.grid)
  # Assign an ID to grid points based on row
  temp.count1 <- as.numeric(temp.grid2@cells.dim[1])
  temp.count2 <- as.numeric(temp.grid2@cells.dim[2])

  # not correct counting rows n-s not e-w)
  rowID <- rep(1:temp.count2, each = temp.count1)

  temp.grid3 <- as.data.frame(temp.grid)
  temp.grid3 <- cbind(temp.grid3, rowID)

  # rotate grid (or rotate polylines)
  xy <- temp.grid3[, 1:2]
  temp.grid4 <- SpatialPointsDataFrame(coords = xy, data = temp.grid3, proj4string = temp.shp@proj4string)

  # ensure that row orientation is included in 'elide' function
  if (rowOrientation != 0) {
    temp.grid4 <- elide(temp.grid4, rotate = (rowOrientation - 90), center = c((gCentroid(coords.bb)$x), (gCentroid(coords.bb)$y)))
    proj4string(temp.grid4) <- temp.shp@proj4string
    # proj4string(temp.grid4)<- CRS("+proj=utm +zone=18 +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0")
  }
  xy2 <- as.data.frame(cbind(temp.grid4$x, temp.grid4$y))


  # CLEAN UP OF ROTATED GRID (WORKS BUT MAYBE NEEDS SOME OPTIMISING)
  temp.clip <- over(temp.grid4, temp.shp)
  temp.clip2 <- cbind(rowID, xy2, temp.clip)
  temp.clip2 <- temp.clip2[, 1:3]
  TotalRows <- max(temp.clip2$rowID)
  temp.clip3 <- temp.clip2[complete.cases(temp.clip2), ]
  temp.clip4 <- temp.clip3
  temp.clip4$rowID <- temp.clip4$rowID - ((min(temp.clip4$rowID - 1)))
  xy3 <- temp.clip4[, 2:3]
  temp.clip4 <- SpatialPointsDataFrame(coords = (xy3), data = temp.clip4, proj4string = temp.shp@proj4string)

  RotatedGrid <- temp.clip4


  # at this point we have a rotate grid aligned with row orientation
  # Extract raster values and coordinates.
  test <- raster::extract(Canopy, SpatialPoints(RotatedGrid), sp = T)
  test <- cbind(test, RotatedGrid$rowID)
  colnames(test@data) <- c("VI", "rowID")

  # remove any NAs in dataframe
  test.df <- as.data.frame(test)
  test.df <- na.omit(test.df)
  # test.df$rowID<-as.character(test.df$rowID)

  # remove edge rows based ob bufferRows
  test.df <- subset(test.df, rowID > (min(test.df$rowID) + bufferRows))
  test.df <- subset(test.df, rowID < (max(test.df$rowID) - bufferRows))
  # temp.clip4<-subset(temp.clip4, rowID<(max(temp.clip4$rowID)-bufferRows))

  # remove short rows that have less than 30 vines (grid points) in them

  for (i in min(test.df$rowID):max(test.df$rowID))
  {
    vines.row <- subset(test.df, rowID == i)
    if (nrow(vines.row) < 30) test.df <- subset(test.df, rowID != i) else test.df <- test.df
  }

  # Calculate variance of each row
  var.start <- 0
  row_sample <- min(test.df$rowID)
  row_var <- NULL
  test.df.tr <- NULL

  for (i in min(test.df$rowID):max(test.df$rowID))
  {
    temp.row <- as.data.frame(subset(test.df, rowID == i))
    temp.row <- temp.row[bufferVines:(nrow(temp.row) - bufferVines), ]
    var.row <- var(temp.row$VI)
    data <- cbind(i, var.row)
    row_var <- rbind(row_var, data)
    test.df.tr <- rbind(test.df.tr, temp.row)
    # if (var.row > var.start) row_sample<-i else row_sample<-row_sample
    # if (var.row > var.start) var.start<-var.row else var.start<-var.start
    # print(i)
  }

  Row_Select <- as.data.frame(row_var)

  # ORDER THE DATAFRAME ON VARIANCE
  Row_Select2 <- Row_Select[order(-Row_Select[, 2]), ]

  # ELIMINATES THE 2 AJOINING ROWS FROM THE SAMPLING
  for (i in 1:maxRowSampled)
  {
    val <- Row_Select2[i, 1]
    Row_Select3 <- subset(Row_Select2, Row_Select2[, 1] != (val + 1))
    Row_Select4 <- subset(Row_Select3, Row_Select3[, 1] != (val + 2))
    Row_Select5 <- subset(Row_Select4, Row_Select4[, 1] != (val - 1))
    Row_Select6 <- subset(Row_Select5, Row_Select5[, 1] != (val - 2))

    Row_Select2 <- Row_Select6
    # print(i)
  }

  # select vector of row numbers to  perform latin sampling
  rowIDs <- Row_Select2[1:maxRowSampled, 1]

  # EXTRACT the TARGET row data from the raster extracted info
  lhs.test <- subset(test.df.tr, rowID %in% rowIDs)

  SampSites_Final <- clhs(lhs.test, size = SampSizeSpec, iter = 200, progress = FALSE, simple = FALSE)
  SampSites_Final2 <- SampSites_Final$sampled_data
  SampSites.SPDF <- SpatialPointsDataFrame(coords = SampSites_Final2[, 3:4], SampSites_Final2, proj4string = temp.shp@proj4string)

  return(SampSites.SPDF)
}

# TESTS
if (F) {
  # library(sp)
  # library(gstat)
  # library(plyr)
  library(rgdal)
  ## library(geoR)
  library(maptools)
  library(rgeos)
  library(raster)
  library(clhs)

  # Inherit parameters from "Vine Plot" window
  rowwidth <- 2.74
  rowOrientation <- 359

  # Specify sampling parameters
  maxRowSampled <- 3 # maximum number of rows to traverse
  SampSizeSpec <- 15 # Desired number of samples

  bufferRows <- 4 # Default value of 4 but option to adjust for the user
  bufferVines <- 6 # Default value of 6 but option to adjust for the user

  # INHERIT SHAPEFILE FROM "VINE PLOT"
  # note - imported shapefile here
  temp.shp <- readOGR(dsn = "C:/Users/jean-luc.lablee/Documents/data/geovit/NDVI_Script/Raw_Input", layer = "Test_DualSensor-canopy-trimmed-data")
  # coordinate system should already be specified.
  # temp.shp<-spTransform(temp.shp,CRS("+proj=utm +zone=17 +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0"))

  # INHERIT raster FROM INTERPOLATION PROCESS (YIELD OR CANOPY) or OPEN DIRECTLY
  # note - imported raster here
  Canopy <- raster(x = "C:/Users/jean-luc.lablee/Documents/data/geovit/NDVI_Script/Raw_Input/Test_DualSensor-canopy-interpolation.tif")

  SampSites.SPDF <- sampling(temp.shp, Canopy, rowwidth, rowOrientation, SampSizeSpec, maxRowSampled, bufferRows, bufferVines)
  writeOGR(SampSites.SPDF, dsn = "C:/Users/jean-luc.lablee/Documents/data/geovit/NDVI_Script/Raw_Input", layer = "Test_DualSensor-canopy-sampling", driver = "ESRI Shapefile", overwrite_layer = T)


  # COMMANDS TO VISUALISE DATA IN RSTUDIO - TO BE REMOVED
  par(mar = rep(2, 4))
  plot(Canopy)
  plot(temp.shp, add = TRUE)
  plot(SampSites.SPDF, bg = "transparent", add = TRUE, pch = 8)
}
