
perform_yield_cleanup <- function(yield_data_frame, Sensor_type, field_name, trim, Delay, summaryFile) {
  temp.df.tr <- as.data.frame(yield_data_frame)

  if (Sensor_type == 1) {
    if ("DistanceTraveled" %in% colnames(temp.df.tr)) {
      temp.df.tr <- temp.df.tr[, c("Longitude", "Latitude", "Yield", "Lin_Yield", "DistanceTraveled")]
      colnames(temp.df.tr) <- c("Longitude", "Latitude", "Yield", "Lin_Yield", "Distance")
    }
    else {
      temp.df.tr <- temp.df.tr[, c("Longitude", "Latitude", "Yield", "Lin_Yield", "Distance")]
    }

    if (Delay > 0) {
      temp.df.tr2 <- cbind(temp.df.tr[1:(nrow(temp.df.tr) - Delay), 1:2], temp.df.tr[(Delay + 1):nrow(temp.df.tr), 3:ncol(temp.df.tr)])
    } else {
      temp.df.tr2 <- cbind(temp.df.tr[(abs(Delay) + 1):nrow(temp.df.tr), 1:2], temp.df.tr[1:(nrow(temp.df.tr) + Delay), 3:ncol(temp.df.tr)])
    }

    temp.df.tr <- subset(temp.df.tr2, Yield > 0.5 & Yield < 25)
    temp.df.tr <- subset(temp.df.tr, Distance > 0.5 & Distance < 5)
    temp.df.tr <- subset(temp.df.tr, Yield > (mean(temp.df.tr$Yield) - trim * sd(temp.df.tr$Yield)) & Yield < (mean(temp.df.tr$Yield) + trim * sd(temp.df.tr$Yield)))
  }

  if (Sensor_type == 2) {
    temp.df.tr <- temp.df.tr[, c("Longitude", "Latitude", "Yld.Mass.Dry..ton.US..ac.", "Distance.ft.")]
    colnames(temp.df.tr) <- c("Longitude", "Latitude", "Yield", "Distance")

    if (Delay > 0) {
      temp.df.tr2 <- cbind(temp.df.tr[1:(nrow(temp.df.tr) - Delay), 1:2], temp.df.tr[(Delay + 1):nrow(temp.df.tr), 3:ncol(temp.df.tr)])
    } else {
      temp.df.tr2 <- cbind(temp.df.tr[(abs(Delay) + 1):nrow(temp.df.tr), 1:2], temp.df.tr[1:(nrow(temp.df.tr) + Delay), 3:ncol(temp.df.tr)])
    }

    temp.df.tr <- subset(temp.df.tr, Yield > 0.5 & Yield < 25)
    temp.df.tr <- subset(temp.df.tr, Distance > 4 & Distance < 12)
    temp.df.tr <- subset(temp.df.tr, Yield > (mean(temp.df.tr$Yield) - trim * sd(temp.df.tr$Yield)) & Yield < (mean(temp.df.tr$Yield) + trim * sd(temp.df.tr$Yield)))
  }

  if (Sensor_type == 3) {
    colnames(temp.df.tr) <- c("Longitude", "Latitude", "Yield")

    if (Delay > 0) {
      temp.df.tr2 <- cbind(temp.df.tr[1:(nrow(temp.df.tr) - Delay), 1:2], temp.df.tr[(Delay + 1):nrow(temp.df.tr), 3:ncol(temp.df.tr)])
    } else {
      temp.df.tr2 <- cbind(temp.df.tr[(abs(Delay) + 1):nrow(temp.df.tr), 1:2], temp.df.tr[1:(nrow(temp.df.tr) + Delay), 3:ncol(temp.df.tr)])
    }

    temp.df.tr <- subset(temp.df.tr, Yield > 0.5 & Yield < 25)
    temp.df.tr <- subset(temp.df.tr, Yield > (mean(temp.df.tr$Yield) - trim * sd(temp.df.tr$Yield)) & Yield < (mean(temp.df.tr$Yield) + trim * sd(temp.df.tr$Yield)))
  }

  temp.df.del <- anti_join(temp.df.tr2, temp.df.tr)

  if(missing(summaryFile))
    summaryFile <- tempfile(pattern = "yield-cleanup-summary-", fileext = ".pdf")
  pdf(file = summaryFile)

  tryCatch({
    par(font.axis = 1, font.lab = 1, font.main = 2, family = "sans", oma = c(0, 0, 3, 0))
    layout(matrix(c(1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5), 2, 12, byrow = TRUE))

    plot(yield_data_frame@coords[, 1], yield_data_frame@coords[, 2], xlab = "Eastings (m)", ylab = "Northings (m)", main = "Raw Data Points", pch = 20, cex = .5)
    plot(temp.df.tr$Longitude, temp.df.tr$Latitude, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Trimmed Data Points", pch = 20, cex = .5)
    plot(temp.df.del$Longitude, temp.df.del$Latitude, xlab = "Eastings (m)", ylab = "Northings (m)", main = "Removed Points", pch = 20, cex = .5)
    hist(yield_data_frame$Yield, xlab = "", ylab = "", main = "Yield (Raw)")
    hist(temp.df.tr$Yield, xlab = "", ylab = "", main = "Yield Trimmed")

    mtext(paste(field_name, " - ", Sys.Date()), outer = TRUE, cex = 1.5)
  },
  error = function(error) stop(error$message),
  finally = dev.off()
  )

  # build trimmed SpatialPointsDataFrame
  coordinates(temp.df.tr) <- c("Longitude", "Latitude")
  proj4string(temp.df.tr) <- CRS(proj4string(yield_data_frame))

  # build removed SpatialPointsDataFrame
  coordinates(temp.df.del) <- c("Longitude", "Latitude")
  proj4string(temp.df.del) <- CRS(proj4string(yield_data_frame))

  list(summaryFile = summaryFile, trimmedDataFrame = temp.df.tr, removedDataFrame = temp.df.del)
}
