
get_yield_file_separator <- function(yield_file) {
  first_line <- readLines(yield_file, n = 1)
  for (separator in c(",", "\t", " ")) {
    if (grepl(separator, first_line)) {
      return(separator)
    }
  }
  stop(paste("Unknown separator in yield file", yield_file))
}

get_yield_sensor_type <- function(yield_file) {
  first_line <- readLines(yield_file, n = 1)
  separator <- get_yield_file_separator(yield_file)
  count_fields <- count.fields(textConnection(first_line), sep = separator)

  if ("Lin_Yield" %in% unlist(strsplit(first_line, separator))) { # ATV SENSOR
    return(1)
  } else if (count_fields == 35) { # Ag Leader SENSOR
    return(2)
  } else if (count_fields == 3) { # Generic data lat-Long-Yield
    return(3)
  } else {
    stop(paste("Unknown yield file type", yield_file))
  }
}

get_yield_data_frame <- function(yield_file, crs) {
  sensor_type <- get_yield_sensor_type(yield_file)
  separator <- get_yield_file_separator(yield_file)
  data <- read.csv(file = yield_file, header = TRUE, sep = separator, stringsAsFactors = FALSE)

  ## Ensure the Lat/Long and Sensor cols contain no strange values
  # All columns as numeric.
  for (d in 1:2) {
    data[, d] <- as.numeric(as.character(data[, d]))
  }
  # All rows with missing/errorneous data are removed.
  for (d in 1:2) {
    data <- subset(data, data[, d] != "NA")
  }

  if (sensor_type == 1) data <- rename(data, Longitude = Lon, Latitude = Lat)
  if (sensor_type == 3) colnames(data) <- c("Longitude", "Latitude", "Yield")

  coordinates(data) <- c("Longitude", "Latitude")
  proj4string(data) <- CRS("+proj=longlat +datum=WGS84")

  if(missing(crs))
      crs <- get_epsg_crs(c(data$Longitude[1], data$Latitude[1]))
  spTransform(data, crs)
}
