
auto_interpolation <- function(data_frame, border, cellsize, attribute_name, cut_border = TRUE, ...) {
  if (!is(data_frame, "SpatialPointsDataFrame")) stop("data_frame must be a SpatialPointsdataFrame")
  if (!is(border, "SpatialPolygons")) stop("border must be a SpatialPolygons")
  if (length(border@polygons) != 1) stop("border must contain only one polygon")
  if (length(border@polygons[[1]]) != 1) stop("border must contain only one polygon")
  if (!identical_coordinate_system(data_frame, border)) stop("data_frame must have identical CRS than the border")

  grid_topology <- get_grid_topology(border, cellsize)
  grid_topology <- increase_grid_topology(grid_topology, 1)
  spatial_points <- SpatialPoints(grid_topology, proj4string = CRS(proj4string(border)))
  spatial_pixels <- SpatialPixels(spatial_points)
  border_lines <- as(border, "SpatialLines")
  if (cut_border) {
    spatial_pixels <- spatial_pixels[!is.na(over(spatial_pixels, border)) | !is.na(over(spatial_pixels, border_lines)), ]
  }
  formula <- as.formula(paste0(attribute_name, "~1"))
  auto_krige(formula, data_frame, spatial_pixels, ...)
}
