
geofis_interpolation = function(x, y, data, cellsize, contour_x, contour_y, model, maxdist, idp, nmin, nmax) {

	# df = pts de depart
	# cellsize = taille de cellule
	# contour_x, contour_y = sommets du polygone du contour de la grille
	
	df = data.frame(x, y, data) 
	coordinates(df) = ~x + y
	
	df_contour = data.frame(contour_x, contour_y)
	coordinates(df_contour) = ~contour_x + contour_y 
	
	bb = bbox(df_contour)
	bb[1,1] = bb[1,1] - cellsize
	bb[1,2] = bb[1,2] + cellsize
	bb[2,1] = bb[2,1] - cellsize
	bb[2,2] = bb[2,2] + cellsize
	
	grid = spsample(df, bb = bb, n = length(df), "regular", cellsize = cellsize, offset = c(0.5, 0.5)) 
	nbc = sum(as.data.frame(grid)[,2] == as.data.frame(grid)[1,2])
	nbr = as.integer(length(grid)/nbc)

	# masque de contour sur la grille
	mask = point.in.polygon(as.data.frame(grid)$x1, as.data.frame(grid)$x2, contour_x, contour_y, mode=T)
	
	# supprimer les points hors du contour
	gridin = grid[as.logical(mask),]
	
	# tres important: supprimer lignes redondantes dans df sinon plantage
	if (!missing(model))
	   	kri = krige(data~1, remove.duplicates(df), gridin, model = model, maxdist = maxdist, nmin = nmin, nmax = nmax)
	else
		kri = idw(data~1, remove.duplicates(df), gridin, maxdist = maxdist, idp = idp)

    xkri<-as.data.frame(grid)$x1
	ykri<-as.data.frame(grid)$x2   
 	zkri<-rep(NA, length(xkri))
    zkri[as.logical(mask)]<-kri$var1.pred
        
    return(list(nbr, nbc, cellsize, xkri, ykri, zkri))
}

