
canopy_process <- R6Class("canopy_process",
  private = list(
    .canopy_file = NULL,
    .canopy_data_frame = NULL,
    .sensor_offset = NULL,
    .speed_range = NULL,
    .swath_width = NULL,
    .ndvi_range = NULL,
    .plot_process = NULL,
    .cleanup_process = NULL,
    .interpolation_process = NULL
  ),

  active = list(
    canopy_file = function(canopy_file) {
      if (missing(canopy_file)) {
        private$.canopy_file
      } else if (is.null(canopy_file)) {
        private$.canopy_file <- NULL
        private$.canopy_data_frame <- NULL
      } else {
        if (!isFile(canopy_file)) stop("canopy file must be a file")
        private$.canopy_file <- canopy_file
        canopy_data_frame <- get_canopy_data_frame(private$.canopy_file, private$.plot_process$crs)
        border <- private$.plot_process$border
        if (!identical_coordinate_system(border, canopy_data_frame)) stop("canopy file must have same coordinate system than the border")
        canopy_data_frame <- get_bounded(canopy_data_frame, border)
        if (nrow(canopy_data_frame) == 0) stop("canopy file must have points inside the border")
        private$.canopy_data_frame <- canopy_data_frame
      }
    },

    canopy_data_frame = function() private$.canopy_data_frame,

    sensor_offset = function(sensor_offset) {
      if (missing(sensor_offset)) {
        private$.sensor_offset
      } else {
        if (!is.numeric(sensor_offset)) stop("sensor offset must be numeric")
        private$.sensor_offset <- sensor_offset
      }
    },

    speed_range = function(speed_range) {
      if (missing(speed_range)) {
        private$.speed_range
      } else {
        if (!is.numeric(speed_range)) stop("speed range must be numeric")
        private$.speed_range <- speed_range
      }
    },

    swath_width = function(swath_width) {
      if (missing(swath_width)) {
        private$.swath_width
      } else {
        if (!is.numeric(swath_width)) stop("swath width must be numeric")
        private$.swath_width <- swath_width
      }
    },

    ndvi_range = function(ndvi_range) {
      if (missing(ndvi_range)) {
        private$.ndvi_range
      } else {
        if (!is.numeric(ndvi_range)) stop("ndvi range must be numeric")
        if (!is.vector(ndvi_range)) stop("ndvi range must be a vector")
        if (!ndvi_range[2] > ndvi_range[1]) stop("ndvi range[2] must be > range[1]")
        if (!ndvi_range["max"] > ndvi_range["min"]) stop("ndvi range max must be > min")
        private$.ndvi_range <- ndvi_range
      }
    },

    cleanup_process = function(cleanup_process) {
      if (missing(cleanup_process)) {
        private$.cleanup_process
      } else {
        private$.cleanup_process <- cleanup_process
      }
    },

    interpolation_process = function(interpolation_process) {
      if (missing(interpolation_process)) {
        private$.interpolation_process
      } else {
        private$.interpolation_process <- interpolation_process
      }
    }
  ),

  public = list(
    initialize = function(output_path, plot_process) {
      private$.plot_process <- plot_process
      private$.cleanup_process <- canopy_cleanup_process$new(output_path, plot_process, self)
      private$.interpolation_process <- canopy_interpolation_process$new(output_path, plot_process, self, private$.cleanup_process)
    },

    release = function() {
      private$.cleanup_process$release()
      private$.interpolation_process$release()
    }
  )
)
