
cleanup_process <- R6Class("cleanup_process",
  private = list(
    .output_path = NULL,
    .summary_file = NULL,
    .trimmed_layer = NULL,
    .trimmed_shapefile = NULL,
    .trimmed_data_frame = NULL,
    .removed_layer = NULL,
    .removed_shapefile = NULL,
    .removed_data_frame = NULL,

    cleanup = function(cleanup_function) {
      cleanup <- cleanup_function()
      private$.trimmed_data_frame <- cleanup$trimmedDataFrame
      private$.removed_data_frame <- cleanup$removedDataFrame
      writeOGR(private$.trimmed_data_frame, private$.output_path, private$.trimmed_layer, driver = "ESRI Shapefile", overwrite_layer = TRUE)
      writeOGR(private$.removed_data_frame, private$.output_path, private$.removed_layer, driver = "ESRI Shapefile", overwrite_layer = TRUE)
    }
  ),

  active = list(
    summary_file = function(summary_file) {
      if (missing(summary_file)) {
        private$.summary_file
      } else {
        private$.summary_file <- summary_file
      }
    },

    trimmed_data_frame = function() private$.trimmed_data_frame,

    removed_data_frame = function() private$.removed_data_frame,

    trimmed_shapefile = function() private$.trimmed_shapefile,

    removed_shapefile = function() private$.removed_shapefile
  ),

  public = list(
    initialize = function(output_path, prefix) {
      private$.output_path <- output_path
      private$.summary_file <- normalizePath(file.path(output_path, paste(prefix, "cleanup-summary.pdf", sep = "-")), mustWork = F)
      private$.trimmed_layer <- paste(prefix, "cleanup-trimmed", sep = "-")
      private$.trimmed_shapefile <- normalizePath(file.path(output_path, paste0(private$.trimmed_layer, ".shp")), mustWork = F)
      private$.removed_layer <- paste(prefix, "cleanup-removed", sep = "-")
      private$.removed_shapefile <- normalizePath(file.path(output_path, paste0(private$.removed_layer, ".shp")), mustWork = F)
    },

    release = function() {
      private$.trimmed_data_frame <- NULL
      private$.removed_data_frame <- NULL
      unlink(private$.summary_file)
      unlink(file.path(private$.output_path, paste0(private$.trimmed_layer, ".*")))
      unlink(file.path(private$.output_path, paste0(private$.removed_layer, ".*")))
    }
  )
)
