
geovit_process <- R6Class("geovit_process",
  private = list(
    .plot_process = NULL,
    .canopy_process = NULL,
    .yield_process = NULL,
    .sampling_process = NULL
  ),

  active = list(
    plot_process = function(plot_process) {
      if (missing(plot_process)) {
        private$.plot_process
      } else {
        private$.plot_process <- plot_process
      }
    },

    canopy_process = function(canopy_process) {
      if (missing(canopy_process)) {
        private$.canopy_process
      } else {
        private$.canopy_process <- canopy_process
      }
    },

    yield_process = function(yield_process) {
      if (missing(yield_process)) {
        private$.yield_process
      } else {
        private$.yield_process <- yield_process
      }
    },

    sampling_process = function(sampling_process) {
      if (missing(sampling_process)) {
        private$.sampling_process
      } else {
        private$.sampling_process <- sampling_process
      }
    },

    epsg = function(epsg) {
      if (!is.numeric(epsg)) stop("epsg must be numeric")
      crs <- CRS(paste0("+init=epsg:", epsg))
      private$.plot_process$crs <- crs
    }
  ),

  public = list(
    initialize = function(output_path) {
      if (missing(output_path)) {
        output_path <- tempdir()
      }
      if (!isDirectory(output_path)) stop("output_path must be a directory")
      private$.plot_process <- plot_process$new()
      private$.canopy_process <- canopy_process$new(output_path, private$.plot_process)
      private$.yield_process <- yield_process$new(output_path, private$.plot_process)
      private$.sampling_process <- sampling_process$new(output_path, private$.plot_process)
    },

    release = function() {
      private$.sampling_process$release()
      private$.yield_process$release()
      private$.canopy_process$release()
      private$.plot_process$release()
    }
  )
)
