
interpolation_process <- R6Class("interpolation_process",
  private = list(
    .interpolation = NULL,
    .summary_file = NULL,
    .raster_file = NULL,

    interpolate = function(interpolation_function) {
      private$.interpolation <- interpolation_function()
      write_interpolation_summary(private$.interpolation, private$.summary_file)
      write_interpolation_raster(private$.interpolation, private$.raster_file)
    }
  ),

  active = list(
    summary_file = function(summary_file) {
      if (missing(summary_file)) {
        private$.summary_file
      } else {
        private$.summary_file <- summary_file
      }
    },

    raster_file = function(raster_file) {
      if (missing(raster_file)) {
        private$.raster_file
      } else {
        private$.raster_file <- raster_file
      }
    },

    krige_output = function() {
      if (!is.null(private$.interpolation)) {
        private$.interpolation$krige_output
      } else {
        NULL
      }
    },

    # for test
    raster = function() {
      if (!is.null(private$.interpolation)) {
        raster <- raster(private$.interpolation$krige_output)
        dataType(raster) <- "FLT8S"
        NAvalue(raster) <- NaN
        return(raster)
      } else {
        NULL
      }
    },

    # for test
    brick = function() {
      if (!is.null(private$.interpolation)) {
        brick <- brick(private$.interpolation$krige_output)
        dataType(brick) <- "FLT8S"
        NAvalue(brick) <- NaN
        return(brick)
      } else {
        NULL
      }
    },

    # for test
    stack = function() {
      if (!is.null(private$.interpolation)) {
        stack <- stack(private$.interpolation$krige_output)
        dataType(stack[[1]]) <- "FLT8S"
        dataType(stack[[2]]) <- "FLT8S"
        # dataType(stack[[3]]) <- "FLT8S"
        NAvalue(stack) <- NaN
        return(stack)
      } else {
        NULL
      }
    }
  ),

  public = list(
    initialize = function(output_path, prefix) {
      private$.summary_file <- normalizePath(file.path(output_path, paste(prefix, "interpolation-summary.pdf", sep = "-")), mustWork = F)
      private$.raster_file <- normalizePath(file.path(output_path, paste(prefix, "interpolation-result.tif", sep = "-")), mustWork = F)
    },

    release = function() {
      private$.interpolation <- NULL
      unlink(private$.summary_file)
      unlink(private$.raster_file)
      unlink(paste0(private$.raster_file, ".aux.xml"))
    }
  )
)
