
plot_process <- R6Class("plot_process",
  private = list(
    .crs = NULL,
    .name = NULL,
    .interrow_distance = NULL,
    .row_orientation = NULL,
    .border = NULL
  ),

  active = list(
    crs = function(crs) {
      if (missing(crs)) {
        if (!is.null(private$.crs)) {
          private$.crs
        }
        else {
          CRS(proj4string(private$.border))
        }
      } else {
        if (!is(crs, "CRS")) stop("crs must be a CRS")
        if (!is.projected(crs)) stop("crs must be a projected coordinate system")
        private$.crs <- crs
        if (!is.null(private$.border)) {
          border <- spTransform(private$.border, crs)
        }
      }
    },

    name = function(name) {
      if (missing(name)) {
        private$.name
      } else {
        if (!is.character(name)) stop("name must be character")
        private$.name <- name
      }
    },

    interrow_distance = function(interrow_distance) {
      if (missing(interrow_distance)) {
        private$.interrow_distance
      } else {
        if (!is.numeric(interrow_distance)) stop("interrow distance must be numeric")
        private$.interrow_distance <- interrow_distance
      }
    },

    row_orientation = function(row_orientation) {
      if (missing(row_orientation)) {
        private$.row_orientation
      } else {
        if (!is.numeric(row_orientation)) stop("row orientation must be numeric")
        private$.row_orientation <- row_orientation
      }
    },

    border = function(border) {
      if (missing(border)) {
        private$.border
      } else if (is.null(border)) {
        private$.border <- NULL
      } else {
        if (!is(border, "SpatialPolygons")) stop("border must be a SpatialPolygons")
        if (is.na(proj4string(border))) stop("border must have coordinate system")
        if (!is.projected(border)) { # border is not projected
          if (!is.null(private$.crs)) { # project in private$.crs if not null
            border <- spTransform(border, private$.crs)
          }
          else { # project in UTM zone
            polygons <- border@polygons[[1]]
            polygon <- polygons@Polygons[[1]]
            epsg_crs <- get_epsg_crs(polygon@coords[1, ])
            border <- spTransform(border, epsg_crs)
          }
        } else { # border is projected
          if (!is.null(private$.crs)) { # reproject in private$.crs if not null
            border <- spTransform(border, private$.crs)
          }
        }
        private$.border <- border
      }
    }
  ),

  public = list(
    release = function() {
      private$.border <- NULL
    }
  )
)
