
sampling_process <- R6Class("sampling_process",
  private = list(
    .output_path = NULL,
    .plot_process = NULL,
    .source = NULL,
    .sample_size = NULL,
    .max_row_sampled = NULL,
    .buffer_rows = NULL,
    .buffer_vines = NULL,
    .sampling_data_frame = NULL,
    .sampling_layer = NULL,
    .sampling_shapefile = NULL
  ),

  active = list(
    source = function(source) {
      if (missing(source)) {
        private$.source
      } else {
        if (is.null(source)) {
          private$.source <- NULL
        } else if (is(source, "Raster")) {
          if (!identical_coordinate_system(source, private$.plot_process$border)) stop("source raster must have same coordinate system than the border")
          private$.source <- source
        } else if (is(source, "SpatialPixelsDataFrame")) {
          if (!identical_coordinate_system(source, private$.plot_process$border)) stop("source raster must have same coordinate system than the border")
          private$.source <- raster(source)
        } else {
          stop("source must be Raster or SpatialPixelsDataFrame")
        }
      }
    },

    sample_size = function(sample_size) {
      if (missing(sample_size)) {
        private$.sample_size
      } else {
        if (!is.numeric(sample_size)) stop("sample_size must be numeric")
        private$.sample_size <- sample_size
      }
    },

    max_row_sampled = function(max_row_sampled) {
      if (missing(max_row_sampled)) {
        private$.max_row_sampled
      } else {
        if (!is.numeric(max_row_sampled)) stop("max_row_sampled must be numeric")
        private$.max_row_sampled <- max_row_sampled
      }
    },

    buffer_rows = function(buffer_rows) {
      if (missing(buffer_rows)) {
        private$.buffer_rows
      } else {
        if (!is.numeric(buffer_rows)) stop("buffer_rows must be numeric")
        private$.buffer_rows <- buffer_rows
      }
    },

    buffer_vines = function(buffer_vines) {
      if (missing(buffer_vines)) {
        private$.buffer_vines
      } else {
        if (!is.numeric(buffer_vines)) stop("buffer_vines must be numeric")
        private$.buffer_vines <- buffer_vines
      }
    },

    sampling_data_frame = function() private$.sampling_data_frame,

    sampling_shapefile = function() private$.sampling_shapefile
  ),

  public = list(
    initialize = function(output_path, plot_process) {
      private$.output_path <- output_path
      private$.plot_process <- plot_process
      private$.sampling_layer <- "sampling"
      private$.sampling_shapefile <- normalizePath(file.path(private$.output_path, paste0(private$.sampling_layer, ".shp")), mustWork = F)
    },

    sample = function() {
      private$.sampling_data_frame <- sampling(private$.plot_process$border, private$.source, private$.plot_process$interrow_distance, private$.plot_process$row_orientation, private$.sample_size, private$.max_row_sampled, private$.buffer_rows, private$.buffer_vines)
      writeOGR(private$.sampling_data_frame, private$.output_path, private$.sampling_layer, driver = "ESRI Shapefile", overwrite_layer = TRUE)
    },

    release = function() {
      private$.sampling_data_frame <- NULL
      unlink(file.path(private$.output_path, paste0(private$.sampling_layer, ".*")))
    }
  )
)
