
yield_process <- R6Class("yield_process",
  private = list(
    .yield_file = NULL,
    .yield_data_frame = NULL,
    .trim = NULL,
    .swath_width = NULL,
    .delay = NULL,
    .plot_process = NULL,
    .cleanup_process = NULL,
    .interpolation_process = NULL
  ),

  active = list(
    yield_file = function(yield_file) {
      if (missing(yield_file)) {
        private$.yield_file
      } else if (is.null(yield_file)) {
        private$.yield_file <- NULL
        private$.yield_data_frame <- NULL
      } else {
        if (!isFile(yield_file)) stop("yield file must be a file")
        private$.yield_file <- yield_file
        yield_data_frame <- get_yield_data_frame(private$.yield_file, private$.plot_process$crs)
        border <- private$.plot_process$border
        if (!identical_coordinate_system(border, yield_data_frame)) stop("yield file must have same coordinate system than the border")
        yield_data_frame <- get_bounded(yield_data_frame, border)
        if (nrow(yield_data_frame) == 0) stop("yield file must have points inside the border")
        private$.yield_data_frame <- yield_data_frame
      }
    },

    yield_data_frame = function() private$.yield_data_frame,

    sensor_type = function() get_yield_sensor_type(private$.yield_file),

    trim = function(trim) {
      if (missing(trim)) {
        private$.trim
      } else {
        if (!is.numeric(trim)) stop("trim must be numeric")
        private$.trim <- trim
      }
    },

    swath_width = function(swath_width) {
      if (missing(swath_width)) {
        private$.swath_width
      } else {
        if (!is.numeric(swath_width)) stop("swath width must be numeric")
        private$.swath_width <- swath_width
      }
    },

    delay = function(delay) {
      if (missing(delay)) {
        private$.delay
      } else {
        if (!is.numeric(delay)) stop("delay must be numeric")
        private$.delay <- delay
      }
    },

    cleanup_process = function(cleanup_process) {
      if (missing(cleanup_process)) {
        private$.cleanup_process
      } else {
        private$.cleanup_process <- cleanup_process
      }
    },

    interpolation_process = function(interpolation_process) {
      if (missing(interpolation_process)) {
        private$.interpolation_process
      } else {
        private$.interpolation_process <- interpolation_process
      }
    }
  ),

  public = list(
    initialize = function(output_path, plot_process) {
      private$.plot_process <- plot_process
      private$.cleanup_process <- yield_cleanup_process$new(output_path, plot_process, self)
      private$.interpolation_process <- yield_interpolation_process$new(output_path, plot_process, self, private$.cleanup_process)
    },

    release = function() {
      private$.cleanup_process$release()
      private$.interpolation_process$release()
    }
  )
)
