
setRefClass("Interpolation", 
	fields = c(grid = "numeric", 
		method = "InterpolationMethod"),

    methods = list(
        computeInterpolation = function(datas, attribute, contour) {
            stopifnot(is(datas, "SpatialPointsDataFrame"))
            stopifnot(is.character(attribute))
            stopifnot(is(contour, "SpatialPolygons"))
            stopifnot(length(contour@polygons) == 1)
            stopifnot(length(contour@polygons[[1]]@Polygons) == 1)
            
            datas <- remove.duplicates(datas)
            bb = bbox(contour)
            bb[1,1] = bb[1,1] - grid
            bb[1,2] = bb[1,2] + grid
            bb[2,1] = bb[2,1] - grid
            bb[2,2] = bb[2,2] + grid
            
            samples = spsample(datas, bb = bb, n = length(datas), "regular", cellsize = grid, offset = c(0.5, 0.5)) 
            nbc = sum(samples@coords[,2] == samples@coords[1,2])
            nbr = as.integer(length(samples)/nbc)
            
            # masque de contour sur la grille
            polygon_contour <- contour@polygons[[1]]@Polygons[[1]]
            mask_in_contour = point.in.polygon(samples@coords[,1], samples@coords[,2], polygon_contour@coords[,1], polygon_contour@coords[,2], mode=T)
            
            # supprimer les points hors du contour
            samples_in_contour = samples[as.logical(mask_in_contour),]
            
            kri <- method$computeInterpolation(datas[[attribute]], datas, samples_in_contour)
            
            xkri<-samples@coords[,1]
            ykri<-samples@coords[,2]   
            zkri<-rep(NA, length(xkri))
            zkri[as.logical(mask_in_contour)]<-kri$var1.pred
            
            #return(list(nbr, nbc, grid, xkri, ykri, zkri))
            return(new("InterpolationResult", nrow = nbr, ncol = nbc, grid = grid, xs = xkri, ys = ykri, values =zkri))            
        },
        
        equals = function(other) {
            identical(grid, other$grid) & method$equals(other$method)
        }
    )
)
