
OpportunityProcess <- setRefClass("OpportunityProcess", 
    contains = "AttributeProcess",    
    
	fields = c(machine = "OpportunityMachine", 
		rotation = "numeric",
		rotateSource = "SpatialPointsDataFrame",
		contour = "SpatialPolygons",
		rotateContour = "SpatialPolygons",
        interpolation = "Interpolation",
        interpolationResult = "InterpolationResult",
        threshold = "OptionalNumeric",
		percentage = "OptionalNumeric",
		result = "OpportunityResult"),

    methods = list(
        computeRotateSource = function(center) {
            rotateSource <<- Rotate(source, rotation, center)
        },
        
        computeRotateContour = function(center) {
            rotateContour <<- Rotate(contour, rotation, center)
        },
        
        computeResultMap = function(center) {
            Rotate(result@map, -rotation, center)
        },
        
        computeInterpolation = function() {
            interpolationResult <<- interpolation$computeInterpolation(rotateSource, getAttributeName(attribute), rotateContour)
        },
        
        computeOpportunity = function() {
            TOi <- main_calcul_TOi_pol(interpolationResult@xs, interpolationResult@ys, interpolationResult@values, interpolationResult@nrow, interpolationResult@ncol, machine, percentage, threshold, interpolation$grid, FALSE)
            proj4string(TOi[[6]]) <- CRS(proj4string(source))
            result <<- OpportunityResult(index = TOi[[1]], threshold = TOi[[2]], error = TOi[[3]], siteA = TOi[[4]], siteB = TOi[[5]], map = TOi[[6]])
        },
        
        equals = function(other) {
            callSuper(other) & 
                identical(machine, other$machine) & 
                identical(rotation, other$rotation) & 
                identical(rotateSource, other$rotateSource) & 
                identical(contour, other$contour) & 
                identical(rotateContour, other$rotateContour) & 
                interpolation$equals(other$interpolation) &
                identical(interpolationResult, other$interpolationResult) & 
                identical(threshold, other$threshold) & 
                identical(percentage, other$percentage) & 
                identical(result, other$result)
        }
    )
)

OpportunityProcess = function(source, attribute, interpolation) { 	
    new("OpportunityProcess", source = source, attribute = attribute, rotateSource = source, threshold = NULL, rotation = 0, percentage = 0.2, interpolation = interpolation, result = EmptyOpportunityResult(source = source)) 
}
