
setGeneric(
  name = "Rotate",
  def = function(object, angle = 0, center = c(0, 0)) {
    return(standardGeneric("Rotate"))
  }
)

setGeneric(
  name = "ConvexHull",
  def = function(object) {
    return(standardGeneric("ConvexHull"))
  }
)

setMethod(
  f = "Rotate",
  signature = "matrix",
  definition = function(object, angle, center) {
    dimnames <- dimnames(object)
    cos <- cos(-angle * pi / 180)
    sin <- sin(-angle * pi / 180)
    translation <- matrix(rep(center, nrow(object)), ncol = 2, byrow = TRUE)
    object <- object - translation
    rotated <- cbind(cos * object[, 1] - sin * object[, 2], sin * object[, 1] + cos * object[, 2]) + translation
    dimnames(rotated) <- dimnames
    return(rotated)
  }
)

setMethod(
  f = "Rotate",
  signature = "SpatialPointsDataFrame",
  definition = function(object, angle, center) {
    rotated <- Rotate(object@coords, angle, center)
    object@coords <- rotated
    dimnames <- dimnames(object@bbox)
    object@bbox <- bbox(rotated)
    dimnames(object@bbox) <- dimnames
    proj4string(object) <- CRS(proj4string(object))
    return(object)
  }
)

setMethod(
  f = "Rotate",
  signature = "Polygon",
  definition = function(object, angle, center) {
    Polygon(Rotate(object@coords, angle, center))
  }
)

setMethod(
  f = "Rotate",
  signature = "list",
  definition = function(object, angle, center) {
    lapply(object, function(x) Rotate(x, angle, center))
  }
)

setMethod(
  f = "Rotate",
  signature = "Polygons",
  definition = function(object, angle, center) {
    Polygons(Rotate(object@Polygons, angle, center), object@ID)
  }
)

setMethod(
  f = "Rotate",
  signature = "SpatialPolygons",
  definition = function(object, angle, center) {
    SpatialPolygons(Rotate(object@polygons, angle, center), object@plotOrder, proj4string = CRS(proj4string(object)))
  }
)

setMethod(
  f = "Rotate",
  signature = "SpatialPolygonsDataFrame",
  definition = function(object, angle, center) {
    SpatialPolygonsDataFrame(Sr = SpatialPolygons(Srl = Rotate(object@polygons, angle, center), proj4string = CRS(proj4string(object))), data = object@data)
  }
)

setMethod(
  f = "ConvexHull",
  signature = "SpatialPointsDataFrame",
  definition = function(object) {
    coords <- object@coords
    convexHull <- chull(coords)
    polygonCoords <- coords[c(convexHull, convexHull[1]), ] # closed polygon
    SpatialPolygons(list(Polygons(list(Polygon(polygonCoords)), ID = 1)), proj4string = CRS(proj4string(object)))
  }
)

EmptySpatialPolygonsDataFrame <- function(proj4string, data) {
  SpatialPolygonsDataFrame(Sr = SpatialPolygons(Srl = list(), proj4string = proj4string), data = data)
}
