library(sp)
library(sf)
library(testthat)
source("polygons.R")

# from https://rgeos.r-forge.r-project.org/reference/comment-functions.html
p1 <- Polygon(cbind(x = c(0, 0, 10, 10, 0), y = c(0, 10, 10, 0, 0)), hole = FALSE) # I
p2 <- Polygon(cbind(x = c(3, 3, 7, 7, 3), y = c(3, 7, 7, 3, 3)), hole = TRUE) # H
p8 <- Polygon(cbind(x = c(1, 1, 2, 2, 1), y = c(1, 2, 2, 1, 1)), hole = TRUE) # H
p9 <- Polygon(cbind(x = c(1, 1, 2, 2, 1), y = c(5, 6, 6, 5, 5)), hole = TRUE) # H

p3 <- Polygon(cbind(x = c(20, 20, 30, 30, 20), y = c(20, 30, 30, 20, 20)), hole = FALSE) # I
p4 <- Polygon(cbind(x = c(21, 21, 29, 29, 21), y = c(21, 29, 29, 21, 21)), hole = TRUE) # H

p5 <- Polygon(cbind(x = c(22, 22, 28, 28, 22), y = c(22, 28, 28, 22, 22)), hole = FALSE) # I
p6 <- Polygon(cbind(x = c(23, 23, 27, 27, 23), y = c(23, 27, 27, 23, 23)), hole = TRUE) # H

p7 <- Polygon(cbind(x = c(13, 13, 17, 17, 13), y = c(13, 17, 17, 13, 13)), hole = FALSE) # I

p10 <- Polygon(cbind(x = c(24, 24, 26, 26, 24), y = c(24, 26, 26, 24, 24)), hole = FALSE) # I
p11 <- Polygon(cbind(x = c(24.25, 24.25, 25.75, 25.75, 24.25), y = c(24.25, 25.75, 25.75, 24.25, 24.25)), hole = TRUE) # H

p12 <- Polygon(cbind(x = c(24.5, 24.5, 25.5, 25.5, 24.5), y = c(24.5, 25.5, 25.5, 24.5, 24.5)), hole = FALSE) # I
p13 <- Polygon(cbind(x = c(24.75, 24.75, 25.25, 25.25, 24.75), y = c(24.75, 25.25, 25.25, 24.75, 24.75)), hole = TRUE) # H

lp <- list(p1, p2, p13, p7, p6, p5, p4, p3, p8, p11, p12, p9, p10)
#           1   2    3   4   5   6   7   8   9   10   11  12   13
#           0   1   11   0   6   0   8   0   1   13    0   1    0
#           I   H    H   I   H   I   H   I   H    H    I   H    I
polygons <- Polygons(lp, ID = "1")
commented_polygons <- polygons
comment(commented_polygons) <- "0 1 11 0 6 0 8 0 1 13 0 1 0" # same as rgeos::createPolygonsComment(polygons)

test_that("test sf polygons", {
  sfp <- st_as_sf(SpatialPolygons(list(polygons)))
  spp <- as_Spatial(sfp)@polygons[[1]]
  # polygons are reordered by ring in st_as_sf
  expect_equal(comment(spp), "0 1 1 1 0 0 6 0 8 0 10 0 12")
})


test_that("test make polygons", {
  polygon_sizes <- c(1, 4, 8)
  points_in_polygons <- c(5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5)
  holes <- as.logical(c(0, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1))
  x_coords <- c(p7@coords[, "x"], p1@coords[, "x"], p2@coords[, "x"], p8@coords[, "x"], p9@coords[, "x"], p3@coords[, "x"], p4@coords[, "x"], p5@coords[, "x"], p6@coords[, "x"], p10@coords[, "x"], p11@coords[, "x"], p12@coords[, "x"], p13@coords[, "x"])
  y_coords <- c(p7@coords[, "y"], p1@coords[, "y"], p2@coords[, "y"], p8@coords[, "y"], p9@coords[, "y"], p3@coords[, "y"], p4@coords[, "y"], p5@coords[, "y"], p6@coords[, "y"], p10@coords[, "y"], p11@coords[, "y"], p12@coords[, "y"], p13@coords[, "y"])
  hole_of <- c(0, 0, 1, 1, 1, 0, 1, 0, 3, 0, 5, 0, 7)

  pl <- lapply(polygon_sizes, make_polygons(x_coords, y_coords, points_in_polygons, holes, hole_of))

  expect_equal(length(pl), 3)
  expect_equal(length(pl[[1]]@Polygons), 1)
  expect_equal(length(pl[[2]]@Polygons), 4)
  expect_equal(length(pl[[3]]@Polygons), 8)

  expect_false(pl[[1]]@Polygons[[1]]@hole)

  expect_false(pl[[2]]@Polygons[[1]]@hole)
  expect_true(pl[[2]]@Polygons[[2]]@hole)
  expect_true(pl[[2]]@Polygons[[3]]@hole)
  expect_true(pl[[2]]@Polygons[[4]]@hole)

  expect_false(pl[[3]]@Polygons[[1]]@hole)
  expect_true(pl[[3]]@Polygons[[2]]@hole)
  expect_false(pl[[3]]@Polygons[[3]]@hole)
  expect_true(pl[[3]]@Polygons[[4]]@hole)
  expect_false(pl[[3]]@Polygons[[5]]@hole)
  expect_true(pl[[3]]@Polygons[[6]]@hole)
  expect_false(pl[[3]]@Polygons[[7]]@hole)
  expect_true(pl[[3]]@Polygons[[8]]@hole)

  expect_identical(pl[[1]]@Polygons[[1]], p7)

  expect_identical(pl[[2]]@Polygons[[1]], p1)
  expect_identical(pl[[2]]@Polygons[[2]], p2)
  expect_identical(pl[[2]]@Polygons[[3]], p8)
  expect_identical(pl[[2]]@Polygons[[4]], p9)

  expect_identical(pl[[3]]@Polygons[[1]], p3)
  expect_identical(pl[[3]]@Polygons[[2]], p4)
  expect_identical(pl[[3]]@Polygons[[3]], p5)
  expect_identical(pl[[3]]@Polygons[[4]], p6)
  expect_identical(pl[[3]]@Polygons[[5]], p10)
  expect_identical(pl[[3]]@Polygons[[6]], p11)
  expect_identical(pl[[3]]@Polygons[[7]], p12)
  expect_identical(pl[[3]]@Polygons[[8]], p13)

  expect_identical(comment(pl[[1]]), "0")
  expect_identical(comment(pl[[2]]), "0 1 1 1")
  expect_identical(comment(pl[[3]]), "0 1 0 3 0 5 0 7")
})


test_that("test explode polygons", {
  ep <- explode_polygons(polygons)

  expect_equal(length(ep), 6)

  expect_equal(length(ep[[1]]), 4)
  expect_equal(length(ep[[2]]), 1)
  expect_equal(length(ep[[3]]), 2)
  expect_equal(length(ep[[4]]), 2)
  expect_equal(length(ep[[5]]), 2)
  expect_equal(length(ep[[6]]), 2)
})

test_that("test explode commented polygons", {
  ep <- explode_polygons(commented_polygons)

  expect_equal(length(ep), 6)

  expect_equal(length(ep[[1]]), 4)
  expect_equal(length(ep[[2]]), 1)
  expect_equal(length(ep[[3]]), 2)
  expect_equal(length(ep[[4]]), 2)
  expect_equal(length(ep[[5]]), 2)
  expect_equal(length(ep[[6]]), 2)

  expect_identical(ep[[1]][[1]], p1)
  expect_identical(ep[[1]][[2]], p2)
  expect_identical(ep[[1]][[3]], p8)
  expect_identical(ep[[1]][[4]], p9)

  expect_identical(ep[[2]][[1]], p7)

  expect_identical(ep[[3]][[1]], p5)
  expect_identical(ep[[3]][[2]], p6)

  expect_identical(ep[[4]][[1]], p3)
  expect_identical(ep[[4]][[2]], p4)

  expect_identical(ep[[5]][[1]], p12)
  expect_identical(ep[[5]][[2]], p13)

  expect_identical(ep[[6]][[1]], p10)
  expect_identical(ep[[6]][[2]], p11)
})


test_that("test explode commented polygons coords", {
  coords <- explode_polygons_coords(commented_polygons)

  expect_equal(length(coords), 6)

  expect_equal(length(coords[[1]]), 4)
  expect_equal(length(coords[[2]]), 1)
  expect_equal(length(coords[[3]]), 2)
  expect_equal(length(coords[[4]]), 2)
  expect_equal(length(coords[[5]]), 2)
  expect_equal(length(coords[[6]]), 2)

  expect_identical(coords[[1]][[1]], p1@coords)
  expect_identical(coords[[1]][[2]], p2@coords)
  expect_identical(coords[[1]][[3]], p8@coords)
  expect_identical(coords[[1]][[4]], p9@coords)

  expect_identical(coords[[2]][[1]], p7@coords)

  expect_identical(coords[[3]][[1]], p5@coords)
  expect_identical(coords[[3]][[2]], p6@coords)

  expect_identical(coords[[4]][[1]], p3@coords)
  expect_identical(coords[[4]][[2]], p4@coords)

  expect_identical(coords[[5]][[1]], p12@coords)
  expect_identical(coords[[5]][[2]], p13@coords)

  expect_identical(coords[[6]][[1]], p10@coords)
  expect_identical(coords[[6]][[2]], p11@coords)
})
