
# based on http://www.igorexchange.com/node/927
#' see https://en.wikipedia.org/wiki/Universal_Transverse_Mercator_coordinate_system
#' see https://stackoverflow.com/questions/18639967/converting-latitude-and-longitude-points-to-utm
#' see https://gis.stackexchange.com/questions/13291/computing-utm-zone-from-lat-long-point
get_utm_zone <- function(longitude, latitude) {
  # Special zone for Norway
  if (latitude >= 56 && latitude < 64 && longitude >= 3 && longitude < 12) {
    return(32)
  }

  # Special zones for Svalbard and Norway
  if (latitude >= 72 && latitude < 84) {
    if (longitude >= 0 && longitude < 9) {
      return(31)
    } else if (longitude >= 9 && longitude < 21) {
      return(33)
    } else if (longitude >= 21 && longitude < 33) {
      return(35)
    } else if (longitude >= 33 && longitude < 42) {
      return(37)
    }
  }

  (floor((longitude + 180) / 6) %% 60) + 1
}

get_utm_hemisphere <- function(latitude) {
  ifelse(latitude > 0, "north", "south")
}

get_utm_crs <- function(longitude, latitude) {
  zone <- get_utm_zone(longitude, latitude)
  hemisphere <- get_utm_hemisphere(latitude)
  CRS(paste0("+proj=utm +zone=", zone, " +", hemisphere, " +datum=WGS84 +units=m"))
}
